@@ -2,6 +2,7 @@ from .base import (
     ClientBaseWithDataset,
     ClientBaseWithDatastack,
     ClientBase,
+    BaseEncoder,
     _api_versions,
     _api_endpoints,
     handle_response,
@@ -9,8 +10,7 @@ from .base import (
 from .auth import AuthClient
 from .endpoints import annotation_common, annotation_api_versions
 from .infoservice import InfoServiceClientV2
-import requests
-import time
+
 import json
 import numpy as np
 from datetime import date, datetime
@@ -20,17 +20,6 @@ from typing import Iterable, Mapping
 SERVER_KEY = "ae_server_address"
 
 
-class AEEncoder(json.JSONEncoder):
-    def default(self, obj):
-        if isinstance(obj, np.ndarray):
-            return obj.tolist()
-        if isinstance(obj, np.uint64):
-            return int(obj)
-        if isinstance(obj, (datetime, date)):
-            return obj.isoformat()
-        return json.JSONEncoder.default(self, obj)
-
-
 def AnnotationClient(
     server_address,
     dataset_name=None,
@@ -38,6 +27,9 @@ def AnnotationClient(
     auth_client=None,
     api_version="latest",
     verify=True,
+    max_retries=None,
+    pool_maxsize=None,
+    pool_block=None,
 ):
     """Factory for returning AnnotationClient
 
@@ -55,6 +47,12 @@ def AnnotationClient(
         'latest': default to the most recent (current 2)
     verify : str (default : True)
         whether to verify https
+    max_retries : Int or None, optional
+        Set the number of retries per request, by default None. If None, defaults to requests package default.
+    pool_block : Bool or None, optional
+        If True, restricts pool of threads to max size, by default None. If None, defaults to requests package default.
+    pool_maxsize : Int or None, optional
+        Sets the max number of threads in the pool, by default None. If None, defaults to requests package default.
 
     Returns
     -------
@@ -85,6 +83,9 @@ def AnnotationClient(
             SERVER_KEY,
             aligned_volume_name,
             verify=verify,
+            max_retries=max_retries,
+            pool_maxsize=pool_maxsize,
+            pool_block=pool_block,
         )
     else:
         return AnnoClient(
@@ -95,6 +96,9 @@ def AnnotationClient(
             SERVER_KEY,
             dataset_name,
             verify=verify,
+            max_retries=max_retries,
+            pool_maxsize=pool_maxsize,
+            pool_block=pool_block,
         )
 
 
@@ -108,6 +112,9 @@ class AnnotationClientV2(ClientBase):
         server_name,
         aligned_volume_name,
         verify=True,
+        max_retries=None,
+        pool_maxsize=None,
+        pool_block=None,
     ):
         super(AnnotationClientV2, self).__init__(
             server_address,
@@ -116,6 +123,9 @@ class AnnotationClientV2(ClientBase):
             endpoints,
             server_name,
             verify=verify,
+            max_retries=max_retries,
+            pool_maxsize=pool_maxsize,
+            pool_block=pool_block,
         )
 
         self._aligned_volume_name = aligned_volume_name
@@ -388,7 +398,7 @@ class AnnotationClientV2(ClientBase):
 
         response = self.session.post(
             url,
-            data=json.dumps(data, cls=AEEncoder),
+            data=json.dumps(data, cls=BaseEncoder),
             headers={"Content-Type": "application/json"},
         )
         return handle_response(response)
@@ -585,7 +595,7 @@ class AnnotationClientV2(ClientBase):
 
         response = self.session.delete(
             url,
-            data=json.dumps(data, cls=AEEncoder),
+            data=json.dumps(data, cls=BaseEncoder),
             headers={"Content-Type": "application/json"},
         )
         return handle_response(response)
