@@ -1,4 +1,5 @@
 import numpy as np
+import pandas as pd
 import requests
 import datetime
 import time
@@ -11,7 +12,13 @@ from .endpoints import (
     chunkedgraph_endpoints_common,
     default_global_server_address,
 )
-from .base import _api_endpoints, _api_versions, ClientBase, handle_response
+from .base import (
+    _api_endpoints,
+    _api_versions,
+    ClientBase,
+    BaseEncoder,
+    handle_response,
+)
 from .auth import AuthClient
 from typing import Iterable
 from urllib.parse import urlencode
@@ -21,17 +28,6 @@ import networkx as nx
 SERVER_KEY = "cg_server_address"
 
 
-class CGEncoder(json.JSONEncoder):
-    def default(self, obj):
-        if isinstance(obj, np.ndarray):
-            return obj.tolist()
-        if isinstance(obj, np.uint64):
-            return int(obj)
-        if isinstance(obj, (datetime.datetime, datetime.date)):
-            return obj.isoformat()
-        return json.JSONEncoder.default(self, obj)
-
-
 def package_bounds(bounds):
     if bounds.shape != (3, 2):
         raise ValueError(
@@ -45,14 +41,39 @@ def package_bounds(bounds):
     return bounds_str
 
 
-def package_timestamp(timestamp):
+def package_timestamp(timestamp, name="timestamp"):
     if timestamp is None:
-        query_d = None
+        query_d = {}
     else:
-        query_d = {"timestamp": time.mktime(timestamp.timetuple())}
+        if timestamp.tzinfo is None:
+            timestamp = pytz.UTC.localize(timestamp)
+        else:
+            timestamp = timestamp.astimezone(datetime.timezone.utc)
+
+        query_d = {name: timestamp.timestamp()}
     return query_d
 
 
+def root_id_int_list_check(
+    root_id,
+):
+    if isinstance(root_id, int) or isinstance(root_id, np.uint64):
+        root_id = [root_id]
+    elif isinstance(root_id, str):
+        try:
+            root_id = np.uint64(root_id)
+        except ValueError:
+            raise ValueError(
+                "When passing a string for 'root_id' make sure the string can be converted to a uint64"
+            )
+    elif isinstance(root_id, np.ndarray) or isinstance(root_id, list):
+        root_id = np.unique(root_id).astype(np.uint64)
+    else:
+        raise ValueError("root_id has to be list or uint64")
+
+    return root_id
+
+
 def ChunkedGraphClient(
     server_address=None,
     table_name=None,
@@ -60,6 +81,9 @@ def ChunkedGraphClient(
     api_version="latest",
     timestamp=None,
     verify=True,
+    max_retries=None,
+    pool_maxsize=None,
+    pool_block=None,
 ):
     if server_address is None:
         server_address = default_global_server_address
@@ -88,6 +112,9 @@ def ChunkedGraphClient(
         timestamp=timestamp,
         table_name=table_name,
         verify=verify,
+        max_retries=max_retries,
+        pool_maxsize=pool_maxsize,
+        pool_block=pool_block,
     )
 
 
@@ -104,6 +131,9 @@ class ChunkedGraphClientV1(ClientBase):
         timestamp=None,
         table_name=None,
         verify=True,
+        max_retries=None,
+        pool_maxsize=None,
+        pool_block=None,
     ):
         super(ChunkedGraphClientV1, self).__init__(
             server_address,
@@ -112,6 +142,9 @@ class ChunkedGraphClientV1(ClientBase):
             endpoints,
             server_key,
             verify=verify,
+            max_retries=max_retries,
+            pool_maxsize=pool_maxsize,
+            pool_block=pool_block,
         )
         self._default_url_mapping["table_id"] = table_name
         self._default_timestamp = timestamp
@@ -228,8 +261,38 @@ class ChunkedGraphClientV1(ClientBase):
         url = self._endpoints["change_log"].format_map(endpoint_mapping)
         params = {"filtered": filtered}
         response = self.session.get(url, params=params)
+
         return handle_response(response)
 
+    def get_tabular_change_log(self, root_ids, filtered=True):
+        """Get a detailed changelog for neurons
+
+        Parameters
+        ----------
+        root_ids : list of np.uint64
+            Object root ids to look up
+
+        Returns
+        -------
+            dict of dataframe
+        """
+        root_ids = [int(r) for r in np.unique(root_ids)]
+
+        endpoint_mapping = self.default_url_mapping
+        endpoint_mapping["root_ids"] = root_ids
+        url = self._endpoints["tabular_change_log"].format_map(endpoint_mapping)
+        params = {"filtered": filtered}
+        data = json.dumps({"root_ids": root_ids}, cls=BaseEncoder)
+
+        response = self.session.get(url, data=data, params=params)
+        res_dict = handle_response(response)
+
+        changelog_dict = {}
+        for k in res_dict.keys():
+            changelog_dict[int(k)] = pd.DataFrame(json.loads(res_dict[k]))
+
+        return changelog_dict
+
     def get_leaves(self, root_id, bounds=None, stop_layer: int = None):
         """Get all supervoxels for a root_id
 
@@ -273,12 +336,13 @@ class ChunkedGraphClientV1(ClientBase):
 
         data = []
         for svid, coor in zip(supervoxels, coords):
-            row = np.concatenate([[svid], np.array(coor) * resolution])
+            pos = np.array(coor) * resolution
+            row = [svid, pos[0], pos[1], pos[2]]
             data.append(row)
         params = {"priority": False}
         response = self.session.post(
             url,
-            data=json.dumps(data, cls=CGEncoder),
+            data=json.dumps(data, cls=BaseEncoder),
             params=params,
             headers={"Content-Type": "application/json"},
         )
@@ -355,7 +419,7 @@ class ChunkedGraphClientV1(ClientBase):
 
         response = self.session.post(
             url,
-            data=json.dumps(nodes, cls=CGEncoder),
+            data=json.dumps(nodes, cls=BaseEncoder),
             params=query_d,
             headers={"Content-Type": "application/json"},
         )
@@ -461,18 +525,19 @@ class ChunkedGraphClientV1(ClientBase):
         dict
             Dictionary describing the lineage graph and operations for the root id.
         """
+        root_id = root_id_int_list_check(root_id)
 
         endpoint_mapping = self.default_url_mapping
-        endpoint_mapping["root_id"] = root_id
 
-        data = {}
+        params = {}
         if timestamp_past is not None:
-            data["timestamp_past"] = time.mktime(timestamp_past.timetuple())
+            params.update(package_timestamp(timestamp_past, name="timestamp_past"))
         if timestamp_future is not None:
-            data["timestamp_future"] = time.mktime(timestamp_future.timetuple())
+            params.update(package_timestamp(timestamp_future, name="timestamp_future"))
 
         url = self._endpoints["handle_lineage_graph"].format_map(endpoint_mapping)
-        r = handle_response(self.session.get(url, params=data))
+        data = json.dumps({"root_ids": root_id}, cls=BaseEncoder)
+        r = handle_response(self.session.post(url, data=data, params=params))
 
         if as_nx_graph:
             return nx.node_link_graph(r)
@@ -494,10 +559,13 @@ class ChunkedGraphClientV1(ClientBase):
         np.ndarray
             1d array with all latest successors
         """
+        root_id = root_id_int_list_check(root_id)
+
+        timestamp_past = self.get_root_timestamps(root_id).min()
 
         lineage_graph = self.get_lineage_graph(
             root_id,
-            timestamp_past=self.get_root_timestamps([root_id])[0],
+            timestamp_past=timestamp_past,
             timestamp_future=timestamp_future,
             as_nx_graph=True,
         )
@@ -522,11 +590,14 @@ class ChunkedGraphClientV1(ClientBase):
         np.ndarray
             1d array with all latest successors
         """
+        root_id = root_id_int_list_check(root_id)
+
+        timestamp_future = self.get_root_timestamps(root_id).max()
 
         lineage_graph = self.get_lineage_graph(
             root_id,
             timestamp_past=timestamp_past,
-            timestamp_future=self.get_root_timestamps([root_id])[0],
+            timestamp_future=timestamp_future,
             as_nx_graph=True,
         )
 
@@ -546,18 +617,22 @@ class ChunkedGraphClientV1(ClientBase):
         Returns:
             np.array[np.Boolean]: boolean array of whether these are valid root_ids
         """
+        root_ids = root_id_int_list_check(root_ids)
+
         endpoint_mapping = self.default_url_mapping
         url = self._endpoints["is_latest_roots"].format_map(endpoint_mapping)
 
         if timestamp is None:
             timestamp = self._default_timestamp
         if timestamp is not None:
-            query_d = {"timestamp": time.mktime(timestamp.timetuple())}
+            query_d = package_timestamp(self._process_timestamp(timestamp))
         else:
             query_d = None
         data = {"node_ids": root_ids}
         r = handle_response(
-            self.session.post(url, data=json.dumps(data, cls=CGEncoder), params=query_d)
+            self.session.post(
+                url, data=json.dumps(data, cls=BaseEncoder), params=query_d
+            )
         )
         return np.array(r["is_latest"], np.bool)
 
@@ -573,19 +648,18 @@ class ChunkedGraphClientV1(ClientBase):
         -------
 
         """
+        root_ids = root_id_int_list_check(root_ids)
+
         endpoint_mapping = self.default_url_mapping
         url = self._endpoints["root_timestamps"].format_map(endpoint_mapping)
 
         data = {"node_ids": root_ids}
         r = handle_response(
-            self.session.post(url, data=json.dumps(data, cls=CGEncoder))
+            self.session.post(url, data=json.dumps(data, cls=BaseEncoder))
         )
 
         return np.array(
-            [
-                pytz.UTC.localize(datetime.datetime.fromtimestamp(ts))
-                for ts in r["timestamp"]
-            ]
+            [datetime.datetime.fromtimestamp(ts, pytz.UTC) for ts in r["timestamp"]]
         )
 
     def get_past_ids(self, root_ids, timestamp_past=None, timestamp_future=None):
@@ -606,18 +680,20 @@ class ChunkedGraphClientV1(ClientBase):
             Dict with keys `future_id_map` and `past_id_map`. Each is a dict whose keys are the supplied root ids and whose values
             are the list of related root ids at the past/future time stamp.
         """
+        root_ids = root_id_int_list_check(root_ids)
+
         endpoint_mapping = self.default_url_mapping
 
         params = {}
         if timestamp_past is not None:
-            params["timestamp_past"] = time.mktime(timestamp_past.timetuple())
+            params.update(package_timestamp(timestamp_past, name="timestamp_past"))
         if timestamp_future is not None:
-            params["timestamp_future"] = time.mktime(timestamp_future.timetuple())
+            params.update(package_timestamp(timestamp_future, name="timestamp_future"))
 
         data = {"root_ids": np.array(root_ids, dtype=np.uint64)}
         url = self._endpoints["past_id_mapping"].format_map(endpoint_mapping)
         r = handle_response(
-            self.session.get(url, data=json.dumps(data, cls=CGEncoder), params=params)
+            self.session.get(url, data=json.dumps(data, cls=BaseEncoder), params=params)
         )
 
         # Convert id keys as strings to ints
@@ -649,10 +725,9 @@ class ChunkedGraphClientV1(ClientBase):
             new_roots (np.ndarray): roots that are new in that interval
         """
         endpoint_mapping = self.default_url_mapping
-        params = {
-            "timestamp_past": time.mktime(timestamp_past.timetuple()),
-            "timestamp_future": time.mktime(timestamp_future.timetuple()),
-        }
+        params = package_timestamp(timestamp_past, name="timestamp_past")
+        params.update(package_timestamp(timestamp_future, name="timestamp_future"))
+
         url = self._endpoints["delta_roots"].format_map(endpoint_mapping)
         r = handle_response(self.session.get(url, params=params))
         return np.array(r["old_roots"]), np.array(r["new_roots"])
