@@ -103,6 +103,7 @@ chunkedgraph_endpoints_v1 = {
     "get_roots": pcg_v1 + "/table/{table_id}/roots_binary",
     "merge_log": pcg_v1 + "/table/{table_id}/root/{root_id}/merge_log",
     "change_log": pcg_v1 + "/table/{table_id}/root/{root_id}/change_log",
+    "tabular_change_log": pcg_v1 + "/table/{table_id}/tabular_change_log_many",
     "contact_sites": pcg_v1 + "/table/{table_id}/node/{root_id}/contact_sites",
     "contact_sites_pairwise": pcg_v1
     + "/table/{table_id}/contact_sites_pair/{root_id_1}/{root_id_2}",
@@ -111,7 +112,7 @@ chunkedgraph_endpoints_v1 = {
     "lvl2_graph": pcg_v1 + "/table/{table_id}/node/{root_id}/lvl2_graph",
     "remesh_level2_chunks": pcg_meshing_v1 + "/table/{table_id}/remeshing",
     "get_subgraph": pcg_v1 + "/table/{table_id}/node/{root_id}/subgraph",
-    "handle_lineage_graph": pcg_v1 + "/table/{table_id}/root/{root_id}/lineage_graph",
+    "handle_lineage_graph": pcg_v1 + "/table/{table_id}/lineage_graph_multiple",
     "past_id_mapping": pcg_v1 + "/table/{table_id}/past_id_mapping",
     "operation_details": pcg_v1 + "/table/{table_id}/operation_details",
     "is_latest_roots": pcg_v1 + "/table/{table_id}/is_latest_roots",
@@ -178,7 +179,11 @@ jsonservice_api_versions = {
 auth_common = {}
 
 v1_auth = "{auth_server_address}/auth/api/v1"
-auth_endpoints_v1 = {"refresh_token": v1_auth + "/refresh_token"}
+auth_endpoints_v1 = {
+    "refresh_token": v1_auth + "/refresh_token",
+    "create_token": v1_auth + "/create_token",
+    "get_users": v1_auth + "/user",
+}
 
 auth_api_versions = {
     1: auth_endpoints_v1,
