@@ -8,6 +8,8 @@ from .materializationengine import MaterializationClient
 from .l2cache import L2CacheClient
 from .endpoints import default_global_server_address
 
+DEFAULT_RETRIES = 3
+
 
 class GlobalClientError(Exception):
     pass
@@ -22,6 +24,9 @@ class CAVEclient(object):
         auth_token_key=None,
         auth_token=None,
         global_only=False,
+        max_retries=DEFAULT_RETRIES,
+        pool_maxsize=None,
+        pool_block=None,
     ):
         if global_only or datastack_name is None:
             return CAVEclientGlobal(
@@ -29,6 +34,9 @@ class CAVEclient(object):
                 auth_token_file=auth_token_file,
                 auth_token_key=auth_token_key,
                 auth_token=auth_token,
+                max_retries=max_retries,
+                pool_maxsize=pool_maxsize,
+                pool_block=pool_block,
             )
         else:
             return CAVEclientFull(
@@ -37,6 +45,9 @@ class CAVEclient(object):
                 auth_token_file=auth_token_file,
                 auth_token_key=auth_token_key,
                 auth_token=auth_token,
+                max_retries=max_retries,
+                pool_maxsize=pool_maxsize,
+                pool_block=pool_block,
             )
 
 
@@ -74,6 +85,12 @@ class CAVEclientGlobal(object):
     auth_token : str or None
         Direct entry of an auth token. If None, uses the file arguments to find the token.
         Optional, default is None.
+    max_retries : int or None, optional
+        Sets the default number of retries on failed requests. Optional, by default 2.
+    pool_maxsize : int or None, optional
+        Sets the max number of threads in a requests pool, although this value will be exceeded if pool_block is set to False. Optional, uses requests defaults if None.
+    pool_block: bool or None, optional
+        If True, prevents the number of threads in a requests pool from exceeding the max size. Optional, uses requests defaults (False) if None.
     """
 
     def __init__(
@@ -82,6 +99,9 @@ class CAVEclientGlobal(object):
         auth_token_file=None,
         auth_token_key=None,
         auth_token=None,
+        max_retries=DEFAULT_RETRIES,
+        pool_maxsize=None,
+        pool_block=None,
     ):
         if server_address is None:
             server_address = default_global_server_address
@@ -92,6 +112,9 @@ class CAVEclientGlobal(object):
             auth_token_key=auth_token_key,
             auth_token=auth_token,
         )
+        self._max_retries = max_retries
+        self._pool_maxsize = pool_maxsize
+        self._pool_block = pool_block
 
     def change_auth(self, auth_token_file=None, auth_token_key=None, auth_token=None):
         """Change the authentication token and reset services.
@@ -142,6 +165,9 @@ class CAVEclientGlobal(object):
                 server_address=self.server_address,
                 datastack_name=self.datastack_name,
                 auth_client=self.auth,
+                max_retries=self._max_retries,
+                pool_maxsize=self._pool_maxsize,
+                pool_block=self._pool_block,
             )
         return self._info
 
@@ -149,7 +175,11 @@ class CAVEclientGlobal(object):
     def state(self):
         if self._state is None:
             self._state = JSONService(
-                server_address=self.server_address, auth_client=self.auth
+                server_address=self.server_address,
+                auth_client=self.auth,
+                max_retries=self._max_retries,
+                pool_maxsize=self._pool_maxsize,
+                pool_block=self._pool_block,
             )
         return self._state
 
@@ -157,7 +187,11 @@ class CAVEclientGlobal(object):
     def schema(self):
         if self._schema is None:
             self._schema = SchemaClient(
-                server_address=self.server_address, auth_client=self.auth
+                server_address=self.server_address,
+                auth_client=self.auth,
+                max_retries=self._max_retries,
+                pool_maxsize=self._pool_maxsize,
+                pool_block=self._pool_block,
             )
         return self._schema
 
@@ -216,6 +250,12 @@ class CAVEclientFull(CAVEclientGlobal):
     auth_token : str or None
         Direct entry of an auth token. If None, uses the file arguments to find the token.
         Optional, default is None.
+    max_retries : int or None, optional
+        Sets the default number of retries on failed requests. Optional, by default 2.
+    pool_maxsize : int or None, optional
+        Sets the max number of threads in a requests pool, although this value will be exceeded if pool_block is set to False. Optional, uses requests defaults if None.
+    pool_block: bool or None, optional
+        If True, prevents the number of threads in a requests pool from exceeding the max size. Optional, uses requests defaults (False) if None.
     """
 
     def __init__(
@@ -225,12 +265,18 @@ class CAVEclientFull(CAVEclientGlobal):
         auth_token_file=default_token_file,
         auth_token_key="token",
         auth_token=None,
+        max_retries=DEFAULT_RETRIES,
+        pool_maxsize=None,
+        pool_block=None,
     ):
         super(CAVEclientFull, self).__init__(
             server_address=server_address,
             auth_token_file=auth_token_file,
             auth_token_key=auth_token_key,
             auth_token=auth_token,
+            max_retries=max_retries,
+            pool_maxsize=pool_maxsize,
+            pool_block=pool_block,
         )
 
         self._datastack_name = datastack_name
@@ -268,6 +314,9 @@ class CAVEclientFull(CAVEclientGlobal):
                 table_name=table_name,
                 server_address=self.local_server,
                 auth_client=self.auth,
+                max_retries=self._max_retries,
+                pool_maxsize=self._pool_maxsize,
+                pool_block=self._pool_block,
             )
         return self._chunkedgraph
 
@@ -278,6 +327,9 @@ class CAVEclientFull(CAVEclientGlobal):
                 server_address=self.local_server,
                 aligned_volume_name=self._aligned_volume_name,
                 auth_client=self.auth,
+                max_retries=self._max_retries,
+                pool_maxsize=self._pool_maxsize,
+                pool_block=self._pool_block,
             )
         return self._annotation
 
@@ -290,6 +342,9 @@ class CAVEclientFull(CAVEclientGlobal):
                 datastack_name=self._datastack_name,
                 cg_client=self.chunkedgraph,
                 synapse_table=self.info.get_datastack_info().get("synapse_table", None),
+                max_retries=self._max_retries,
+                pool_maxsize=self._pool_maxsize,
+                pool_block=self._pool_block,
             )
         return self._materialize
 
@@ -300,6 +355,9 @@ class CAVEclientFull(CAVEclientGlobal):
                 server_address=self.server_address,
                 auth_client=self.auth,
                 ngl_url=self.info.viewer_site(),
+                max_retries=self._max_retries,
+                pool_maxsize=self._pool_maxsize,
+                pool_block=self._pool_block,
             )
         return self._state
 
@@ -313,5 +371,8 @@ class CAVEclientFull(CAVEclientGlobal):
                 server_address=self.local_server,
                 auth_client=self.auth,
                 table_name=table_name,
+                max_retries=self._max_retries,
+                pool_maxsize=self._pool_maxsize,
+                pool_block=self._pool_block,
             )
         return self._l2cache
