@@ -9,6 +9,7 @@ from .base import (
     ClientBaseWithDataset,
     ClientBaseWithDatastack,
     ClientBase,
+    BaseEncoder,
     _api_versions,
     _api_endpoints,
     handle_response,
@@ -62,19 +63,6 @@ def concatenate_position_columns(df, inplace=False):
     return df2
 
 
-class MEEncoder(json.JSONEncoder):
-    def default(self, obj):
-        if isinstance(obj, np.ndarray):
-            return obj.tolist()
-        if isinstance(obj, np.uint64):
-            return int(obj)
-        if isinstance(obj, np.int64):
-            return int(obj)
-        if isinstance(obj, (datetime, date)):
-            return obj.isoformat()
-        return json.JSONEncoder.default(self, obj)
-
-
 def convert_timestamp(ts: datetime):
     if isinstance(ts, datetime):
         if ts.tzinfo is None:
@@ -94,6 +82,9 @@ def MaterializationClient(
     api_version="latest",
     version=None,
     verify=True,
+    max_retries=None,
+    pool_maxsize=None,
+    pool_block=None,
 ):
     """Factory for returning AnnotationClient
 
@@ -147,6 +138,9 @@ def MaterializationClient(
         synapse_table=synapse_table,
         version=version,
         verify=verify,
+        max_retries=max_retries,
+        pool_maxsize=pool_maxsize,
+        pool_block=pool_block,
     )
 
 
@@ -163,6 +157,9 @@ class MaterializatonClientV2(ClientBase):
         synapse_table=None,
         version=None,
         verify=True,
+        max_retries=None,
+        pool_maxsize=None,
+        pool_block=None,
     ):
         super(MaterializatonClientV2, self).__init__(
             server_address,
@@ -171,6 +168,9 @@ class MaterializatonClientV2(ClientBase):
             endpoints,
             server_name,
             verify=verify,
+            max_retries=max_retries,
+            pool_maxsize=pool_maxsize,
+            pool_block=pool_block,
         )
         self._datastack_name = datastack_name
         if version is None:
@@ -582,7 +582,7 @@ class MaterializatonClientV2(ClientBase):
 
         response = self.session.post(
             url,
-            data=json.dumps(data, cls=MEEncoder),
+            data=json.dumps(data, cls=BaseEncoder),
             headers={"Content-Type": "application/json", "Accept-Encoding": encoding},
             params=query_args,
             stream=~return_df,
@@ -682,7 +682,7 @@ class MaterializatonClientV2(ClientBase):
 
         response = self.session.post(
             url,
-            data=json.dumps(data, cls=MEEncoder),
+            data=json.dumps(data, cls=BaseEncoder),
             headers={"Content-Type": "application/json", "Accept-Encoding": encoding},
             params=query_args,
             stream=~return_df,
@@ -725,9 +725,9 @@ class MaterializatonClientV2(ClientBase):
                         else:
                             root_ids.append(val)
 
-        # if they are all None then we can safely return now
+        # if there are no root_ids then we can safely return now
         if len(root_ids) == 0:
-            return [None, None, None], {}
+            return filters, {}
         root_ids = np.unique(np.concatenate(root_ids))
 
         filter_timed_end = self.cg_client.is_latest_roots(root_ids, timestamp=timestamp)
@@ -984,7 +984,7 @@ class MaterializatonClientV2(ClientBase):
         with TimeIt("query materialize"):
             response = self.session.post(
                 url,
-                data=json.dumps(data, cls=MEEncoder),
+                data=json.dumps(data, cls=BaseEncoder),
                 headers={
                     "Content-Type": "application/json",
                     "Accept-Encoding": encoding,
